﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml.Linq;
using System.Reflection;
using System.Collections;

namespace Students
{
    class Program
    {
        static void Print(string title, IEnumerable objs)
        {
            Console.WriteLine(title);
            foreach (var o in objs)
            {
                Console.WriteLine(o);
            }
            Console.WriteLine();
        }

        static List<Student> ReadStudents()
        {
            List<Student> students;

            // load resource, file can be opened for example by using stream = File.OpenText(filename)
            using (var stream = Assembly.GetAssembly(typeof(Program)).GetManifestResourceStream("Students.students.xml"))
            {
                var xStudents = XElement.Load(stream);
                students = (from student in xStudents.Elements()
                            select Student.FromXML(student)).ToList();
                // same as students = xStudents.Elements().Select(s => Student.FromXML(s))
            }

            return students;
        }

        static void Main(string[] args)
        {
            var students = ReadStudents();
            
            var males = students.Where(s => s.Gender == Student.Genders.Male);
            Print("Male students:", males);

            var females = students.Where(s => s.Gender == Student.Genders.Female);
            Print("Female students:", females);

            // always works :)
            // implemented as an extension method in Extensions.cs
            students.RandomlyGradeStudents();

            var passed = students.Where(s => s.Grade != Student.Grades.F);
            Print("Passed students:", passed);

            var topTen = passed.OrderByDescending(s => s.Score).Take(10);
            Print("Top 10 passed students:", topTen);

            // poor bastards ... :)
            var ucosOfFailedStudents = students.Where(s => s.Grade == Student.Grades.F).Select(s => s.UCO);
            Console.WriteLine("UCOs of failed students:");
            Console.WriteLine(string.Join(", ", ucosOfFailedStudents));
            Console.WriteLine();

            var teacherNotes = new List<TeacherNote>();

            foreach (var student in topTen)
            {
                teacherNotes.Add(new TeacherNote() { UCO = student.UCO, Note = "Congratulate on making it to the top 10!" });

                /* 
                 * new TeacherNote() { UCO = student.UCO, Note = "Congratulate on making it to the top 10!" }
                 * is equivalent to 
                 * var note = new TeacherNote();
                 * note.UCO = ...;
                 * note.Note = ...;
                 */
            }

            // this code is similar to the previous foreach loop with the difference of using the "Run" extension which wraps the foreach loop
            students.Where(s => s.Grade == Student.Grades.E)
                .Run(s => teacherNotes.Add(new TeacherNote { UCO = s.UCO, Note = "Recommend to spend more time preparing..." }));

            // inner join [http://en.wikipedia.org/wiki/Join_(SQL)] on students and notes + creating an anonymous type
            // anonymous types can be created only inside the body of a function
            var studentsWithNotes = from student in students
                                    join note in teacherNotes on student.UCO equals note.UCO
                                    select new { FullName = student.Name + " " + student.Surname, note.Note };
            /*
             * new { FullName = student.Name + " " + student.Surname, note.Note };
             * is equivalent to creating a class with properties string FullName and string Note
             */

            Console.WriteLine("Students with notes:");
            studentsWithNotes.Run(s => Console.WriteLine("{0}: {1}", s.FullName, s.Note));
            Console.WriteLine();

            // students can now be accessed by their UCO by studentsByUco[uco]
            Dictionary<int, Student> studentsByUco = students.ToDictionary(s => s.UCO, s => s);

            // group the students by their study and print each group
            var studentsGroupedByStudy = students.GroupBy(s => s.Study);
            studentsGroupedByStudy.Run(group => Print(group.Key, group));

            // calculate the average score
            var averageScore = students.Average(s => s.Score);
            Console.WriteLine("The average score is {0:0.00}.", averageScore);

            Console.ReadLine();
        }
    }
}
